from typing import Iterable

from fetchfox import rest

ADA = "000000000000000000000000000000000000000000000000000000006c6f76656c616365"
BASE_URL = "https://dexhunter.sbase.ch/"


def get(service: str, partner_code: str, params: dict = None) -> dict:
    url = f"{BASE_URL}/{service}"

    response, _ = rest.get(
        url=url,
        params=params,
        headers={
            "X-Partner-Id": partner_code,
        },
    )

    return response


def post(service: str, body: dict, partner_code: str, params: dict = None) -> dict:
    url = f"{BASE_URL}/{service}"

    response, _ = rest.post(
        url=url,
        params=params,
        body=body,
        headers={
            "X-Partner-Id": partner_code,
        },
    )

    return response


def get_global_orders(asset_id: str, partner_code: str) -> Iterable[dict]:
    body = {
        "filters": [
            {
                "filterType": "TOKENID",
                "values": [asset_id],
            },
            {
                "filterType": "STATUS",
                "values": ["COMPLETED"],
            },
        ],
        "orderSorts": "STARTTIME",
        "page": 0,
        "perPage": 100,
        "sortDirection": "DESC",
    }

    page = -1

    while True:
        page += 1
        body["page"] = page

        response = post(
            "swap/globalOrders",
            body=body,
            partner_code=partner_code,
        )

        if not response.get("orders"):
            break

        yield from response["orders"]


def get_average_price(asset_id: str, partner_code: str) -> float:
    response = get(
        f"swap/averagePrice/{asset_id}/ADA",
        partner_code=partner_code,
    )

    return response["average_price"]


def get_pair_stats(asset_id: str, partner_code: str) -> dict:
    return get(
        f"swap/pairStats/{asset_id}/ADA",
        partner_code=partner_code,
    )



from fetchfox.apis.evm import openseaio
from fetchfox.blockchains.evm import Evm
from fetchfox.constants.blockchains import POLYGON
from fetchfox.constants.currencies import MATIC


class Polygon(Evm):
    def __init__(
        self,
        moralisio_api_key: str = None,
        openseaio_api_key: str = None,
    ):
        super().__init__(
            name=POLYGON,
            currency=MATIC,
            logo="https://s2.coinmarketcap.com/static/img/coins/64x64/3890.png",
            moralisio_api_key=moralisio_api_key,
            openseaio_api_key=openseaio_api_key,
        )

    def explorer_url(self, *, address: str = None, collection_id: str = None, asset_id: str = None, tx_hash: str = None) -> str:
        if address:
            return f"https://polygonscan.com/address/{address.lower()}"

        if asset_id:
            assert collection_id
            return f"https://polygonscan.com/token/{collection_id.lower()}?a={asset_id}"

        if collection_id:
            return f"https://polygonscan.com/token/{collection_id.lower()}"

        if tx_hash:
            return f"https://polygonscan.com/tx/{tx_hash.lower()}"

        return None

    def marketplace_url(self, *, collection_id: str = None, asset_id: str = None) -> str:
        if asset_id:
            assert collection_id
            return f"https://opensea.io/assets/matic/{collection_id.lower()}/{asset_id}"

        if collection_id:
            slug = openseaio.get_slug(
                contract_address=collection_id,
                blockchain=self.name,
                api_key=self.openseaio_api_key,
            )

            return f"https://opensea.io/collection/{slug}"

        return None



from cachetools.func import ttl_cache

from fetchfox import rest
from fetchfox.checks import check_str

BASE_URL = "https://api.muesliswap.com"


@ttl_cache(ttl=60)
def price(asset_id: str) -> float:
    check_str(asset_id, "muesliswap.asset_id")

    policy_id, asset_name = asset_id[:56], asset_id[56:]

    response, status_code = rest.get(
        url=f"{BASE_URL}/price",
        params={
            "quote-policy-id": policy_id,
            "quote-tokenname": asset_name,
        },
    )

    return response["price"]



from typing import Iterable, Tuple

from fetchfox import rest
from fetchfox.checks import check_str

BASE_URL = "https://api.algoxnft.com"


def get(service: str, params: dict = None, version: int = 1) -> Tuple[dict, int]:
    return rest.get(
        url=f"{BASE_URL}/v{version}/{service}",
        params=params or {},
    )


def get_listings(creator_address: str) -> Iterable[dict]:
    check_str(creator_address, "algoxnftcom.creator_address")
    creator_address = creator_address.strip().upper()

    response, status_code = get(f"nft-explorer/creator/{creator_address}")

    yield from response



from fetchfox.apis import coingeckocom
from fetchfox.apis.cardano import muesliswap
from fetchfox.constants.book import BOOK_TOKEN_ASSET_ID
from fetchfox.constants.currencies import ADA, BOOK


def usd(currency: str) -> float:
    if currency == BOOK:
        ada = muesliswap.price(BOOK_TOKEN_ASSET_ID)

        return ada * coingeckocom.usd(ADA)
    else:
        return coingeckocom.usd(currency)



from typing import Iterable, Tuple

from cachetools.func import ttl_cache

from fetchfox.apis import price
from fetchfox.apis.cardano import cexplorerio, dexhunterio
from fetchfox.constants.book import (
    BOOK_TOKEN_ASSET_ID,
    BOOK_TOKEN_ASSET_NAME,
    BOOK_TOKEN_POLICY_ID,
    BOOK_TOKEN_FINGERPRINT,
)
from fetchfox.constants.currencies import ADA, BOOK
from fetchfox.dtos import OrderDTO, OrderType, PairStatsDTO, StatsDTO


class BookToken:
    asset_id = BOOK_TOKEN_ASSET_ID
    asset_name = BOOK_TOKEN_ASSET_NAME
    fingerprint = BOOK_TOKEN_FINGERPRINT
    policy_id = BOOK_TOKEN_POLICY_ID
    symbol = BOOK

    def __init__(self, dexhunter_partner_code: str):
        self.dexhunter_partner_code = dexhunter_partner_code

    @property
    @ttl_cache(ttl=30)
    def ada(self) -> float:
        return dexhunterio.get_average_price(
            self.asset_id,
            partner_code=self.dexhunter_partner_code,
        )

    @property
    @ttl_cache(ttl=30)
    def usd(self) -> float:
        return price.usd(ADA) * self.ada

    @property
    def cardanoscan_url(self):
        return "https://cardanoscan.io/token/{asset_id}".format(
            asset_id=self.asset_id,
        )

    @property
    def cexplorer_url(self):
        return "https://cexplorer.io/asset/{fingerprint}".format(
            fingerprint=self.fingerprint,
        )

    @property
    def minswap_url(self):
        return "https://app.minswap.org/swap?currencySymbolA={policy_id}&tokenNameA={token_name}&currencySymbolB=&tokenNameB=".format(
            policy_id=self.policy_id,
            token_name=self.asset_name,
        )

    @property
    def owners(self) -> Tuple[int, int]:
        tx = cexplorerio.get_asset_detail(self.fingerprint)

        return tx["epoch"], tx["owners_acc"]

    @property
    def pair_stats(self) -> PairStatsDTO:
        stats = dexhunterio.get_pair_stats(
            self.asset_id,
            partner_code=self.dexhunter_partner_code,
        )

        stats_dto = StatsDTO(
            ada=stats["token2Amount"],
            book=stats["token1Amount"],
            daily_txs=stats["dailyTxAmount"],
            daily_buys=stats["dailyBuysCount"],
            daily_sales=stats["dailySalesCount"],
            daily_volume=stats["dailyVolume"],
            price_change_hour=stats["priceChangeHour"],
            price_change_day=stats["priceChangeDay"],
            price_change_week=stats["priceChangeWeek"],
            price_change_month=stats["priceChangeMonth"],
        )

        pair_stats_dto = PairStatsDTO(stats_dto)

        for pool in stats["pools"]:
            pair_stats_dto.add_pool(
                name=pool["dexName"].lower(),
                stats=StatsDTO(
                    ada=pool["token2Amount"],
                    book=pool["token1Amount"],
                    daily_txs=pool["dailyTxAmount"],
                    daily_buys=pool["dailyBuysCount"],
                    daily_sales=pool["dailySalesCount"],
                    daily_volume=pool["dailyVolume"],
                    price_change_hour=pool["priceChangeHour"],
                    price_change_day=pool["priceChangeDay"],
                    price_change_week=pool["priceChangeWeek"],
                    price_change_month=pool["priceChangeMonth"],
                ),
            )

        return pair_stats_dto

    def last_orders(self, limit: int = 25) -> Iterable[OrderDTO]:
        orders = dexhunterio.get_global_orders(
            self.asset_id,
            partner_code=self.dexhunter_partner_code,
        )

        for index, order in enumerate(orders):
            if index == limit:
                break

            if order["token_id_in"] == dexhunterio.ADA:
                order_type = OrderType.BUY
                ada = order["amount_in"]
                book = order["actual_out_amount"]
            else:
                order_type = OrderType.SELL
                book = order["amount_in"]
                ada = order["actual_out_amount"]

            yield OrderDTO(
                address=order["user_address"],
                ada=ada,
                book=book,
                order_type=order_type,
                dex=order["dex"],
                tx_hash=order.get("update_tx_hash") or order["tx_hash"],
            )



import logging
from functools import lru_cache
from typing import Iterable, Tuple

from fetchfox import rest
from fetchfox.checks import check_str

BASE_URL = "https://deep-index.moralis.io/api"

logger = logging.getLogger(__name__)


def get(service: str, blockchain: str, api_key: str, params: dict = None, version: int = 2) -> Tuple[dict, int]:
    check_str(blockchain, "moralisio.blockchain")
    check_str(api_key, "moralisio.api_key")

    params = params or {}
    params["chain"] = "eth" if blockchain == "ethereum" else blockchain

    return rest.get(
        url=f"{BASE_URL}/v{version}/{service}",
        params=params,
        headers={
            "X-API-Key": api_key,
            "Host": "deep-index.moralis.io",
        },
    )


def get_assets(contract_address: str, blockchain: str, api_key: str) -> Iterable[dict]:
    check_str(contract_address, "moralisio.contract_address")
    contract_address = contract_address.strip().lower()

    cursor = ""

    while True:
        response, status_code = get(
            f"nft/{contract_address}",
            params={
                "cursor": cursor,
                "format": "decimal",
                "normalizeMetadata": "false",
            },
            blockchain=blockchain,
            api_key=api_key,
        )

        for item in response.get("result", []):
            yield item["token_id"]

        if not response.get("cursor"):
            break

        cursor = response["cursor"]


def resync_metadata(contract_address: str, asset_id: str, blockchain: str, api_key: str):
    check_str(contract_address, "moralisio.contract_address")
    check_str(asset_id, "moralisio.asset_id")
    contract_address = contract_address.strip().lower()

    response, status_code = get(
        f"nft/{contract_address}/{asset_id}/metadata/resync",
        params={
            "flag": "uri",
            "mode": "sync",
        },
        blockchain=blockchain,
        api_key=api_key,
    )

    if status_code == 404:
        raise ValueError(f"{contract_address}/{asset_id} doesn't exist")

    return response.get("status")


@lru_cache(maxsize=None)
def get_asset_data(contract_address: str, asset_id: str, blockchain: str, api_key: str) -> dict:
    check_str(contract_address, "moralisio.contract_address")
    check_str(asset_id, "moralisio.asset_id")
    contract_address = contract_address.strip().lower()

    response, status_code = get(
        f"nft/{contract_address}/{asset_id}",
        blockchain=blockchain,
        api_key=api_key,
    )

    if status_code == 404:
        raise Exception(f"{contract_address}/{asset_id} doesn't exist")

    if not response.get("metadata"):
        resync_metadata(
            contract_address,
            asset_id,
            blockchain,
            api_key=api_key,
        )

        return get_asset_data(contract_address, asset_id, blockchain, api_key=api_key)

    return response


def get_balance(address: str, blockchain: str, api_key: str) -> Iterable[dict]:
    check_str(address, "moralisio.address")
    address = address.strip().lower()

    response, status_code = get(
        f"{address}/balance",
        blockchain=blockchain,
        api_key=api_key,
    )

    return int(response["balance"]) / 10**18


def get_holdings(address: str, blockchain: str, api_key: str) -> Iterable[dict]:
    check_str(address, "moralisio.address")
    address = address.strip().lower()

    cursor = ""

    while True:
        response, status_code = get(
            f"{address}/nft",
            params={
                "cursor": cursor,
            },
            blockchain=blockchain,
            api_key=api_key,
        )

        for item in response.get("result", []):
            yield item

        if not response.get("cursor"):
            break

        cursor = response["cursor"]


def get_owners(contract_address: str, blockchain: str, api_key: str) -> Iterable[dict]:
    check_str(contract_address, "moralisio.contract_address")
    contract_address = contract_address.strip().lower()

    cursor = ""

    while True:
        response, status_code = get(
            f"nft/{contract_address}/owners",
            params={
                "cursor": cursor,
            },
            blockchain=blockchain,
            api_key=api_key,
        )

        for holding in response["result"]:
            yield {
                "contract_address": holding["token_address"],
                "asset_id": holding["token_id"],
                "address": holding["owner_of"],
                "amount": holding["amount"],
            }

        cursor = response.get("cursor")

        if not cursor:
            break


def get_owner(contract_address: str, asset_id: str, blockchain: str, api_key: str) -> dict:
    check_str(contract_address, "moralisio.contract_address")
    check_str(asset_id, "moralisio.asset_id")
    contract_address = contract_address.strip().lower()

    response, status_code = get(
        f"nft/{contract_address}/{asset_id}/owners",
        blockchain=blockchain,
        api_key=api_key,
    )

    for holding in response["result"]:
        return {
            "contract_address": holding["token_address"],
            "asset_id": holding["token_id"],
            "address": holding["owner_of"],
            "amount": holding["amount"],
        }



from enum import Enum


class OrderType(str, Enum):
    BUY = "BUY"
    SELL = "SELL"


class OrderDTO:
    def __init__(self, address: str, ada: float, book: float, dex: str, tx_hash: str, order_type: OrderType):
        self.address: str = address
        self.ada: float = ada
        self.book: float = book
        self.dex: str = dex.lower()
        self.tx_hash: str = tx_hash
        self.order_type: OrderType = order_type

    @property
    def average(self) -> float:
        return self.ada / self.book

    def __repr__(self):
        if self.order_type == OrderType.BUY:
            return f"{round(self.ada)} ADA > {round(self.book)} BOOK ({self.dex}) [{self.average}]"
        else:
            return f"{round(self.book)} BOOK > {round(self.ada)} ADA ({self.dex}) [{self.average}]"



import typing

if typing.TYPE_CHECKING:
    from .listing import ListingDTO


class FloorDTO:
    def __init__(
        self,
        listing: "ListingDTO",
        listing_count: int,
    ):
        self.listing: "ListingDTO" = listing
        self.listing_count: int = listing_count



import logging
from functools import lru_cache
from typing import Tuple

from fetchfox import rest
from fetchfox.checks import check_str

BASE_URL = "https://api.ensideas.com"

logger = logging.getLogger(__name__)


def get(service: str, params: dict = None) -> Tuple[dict, int]:
    return rest.get(
        url=f"{BASE_URL}/{service}",
        params=params or {},
    )


@lru_cache(maxsize=None)
def resolve_ens_domain(ens_domain: str):
    check_str(ens_domain)

    response, status_code = get(f"ens/resolve/{ens_domain}")

    address = response.get("address")

    if not address:
        return None

    logger.info("resolved %s to %s", ens_domain, address)
    return address


@lru_cache(maxsize=None)
def get_ens_domain(address: str):
    check_str(address)

    response, status_code = get(f"ens/resolve/{address}")

    ens_domain = response.get("name")

    if not ens_domain:
        return None

    logger.info("resolved %s to %s", address, ens_domain)
    return ens_domain



from .asset import AssetDTO
from .campaign import CampaignDTO
from .floor import FloorDTO
from .holding import HoldingDTO
from .listing import ListingDTO
from .order import OrderDTO, OrderType
from .pair_stats import PairStatsDTO, StatsDTO
from .rank import RankDTO
from .sale import SaleDTO, SaleType
